<?php

namespace WPIS;

require_once __DIR__ . '/../../../../wp-load.php';
class CronManager {

    const IMPORT_LIMIT = 5;

    private $error = false;
    private $action = null;
    private $currentImport = 1;
    private $importSettings = [];
    private $importList = [];
    private $allowedActions = ['import', 'update', 'remove'];
    private $cacheDirectory;

    public function __construct() {

        $this->cacheDirectory = __DIR__ . '/../../../uploads/wp-immo-sync';

        date_default_timezone_set('Europe/Brussels');
        set_error_handler([$this, 'errorHandler'], E_ALL);

        $this->checkAuthorisation();

        if( !$this->error ){

            Logger::log('wpis-cron', '[CRON] ' . strtoupper($this->action) . ' - START');

            switch( $this->action ){

                case 'import':
                    if( SettingsManager::get('enableCronImport') ){
                        $this->importSettings = (object) ['updateOnly' => false, 'forcePictureImport' => false];
                        Logger::log('wpis-cron', '[CRON] IMPORT - START');
                        $result = $this->import();
                    }
                    else {
                        Logger::log('wpis-cron', '[CRON] IMPORT CRON DISABLED');
                        $result = ['message' => 'IMPORT CRON DISABLED'];
                    }
                    break;

                case 'update':
                    if( SettingsManager::get('enableCronUpdate') ){
                        Logger::log('wpis-cron', '[CRON] UPDATE - START');
                        $result = $this->update();
                    }
                    else {
                        Logger::log('wpis-cron', '[CRON] UPDATE CRON DISABLED');
                        $result = ['message' => 'UPDATE CRON DISABLED'];
                    }
                    break;

                case 'remove':
                    if( SettingsManager::get('enableCronRemove') ){
                        Logger::log('wpis-cron', '[CRON] REMOVE - START');
                        $result = $this->remove();
                    }
                    else {
                        Logger::log('wpis-cron', '[CRON] REMOVE CRON DISABLED');
                        $result = ['message' => 'REMOVE CRON DISABLED'];
                    }
                    break;

                case 'auto':
                    $result = [];
                    if( SettingsManager::get('enableCronRemove') ){
                        Logger::log('wpis-cron', '[CRON] REMOVE - START');
                        $result['remove'] = $this->remove();
                    }
                    if( SettingsManager::get('enableCronUpdate') ){
                        Logger::log('wpis-cron', '[CRON] UPDATE - START');
                        $result['update'] = $this->update();
                    }
                    if( SettingsManager::get('enableCronImport') ){
                        Logger::log('wpis-cron', '[CRON] IMPORT - START');
                        $result['import'] = $this->import();
                    }
                    return $this->response([
                        'success' => true,
                        'result'  => $result
                    ]);
                    break;
            }

            return $this->response([
                'success' => true,
                'result' => $result
            ]);
        }
    }

    public function errorHandler($e, $errstr) {

        if( in_array($e, [E_WARNING, E_ERROR]) ){

            if( file_exists($this->cacheDirectory . '/wpis.lock') ){
                unlink($this->cacheDirectory . '/wpis.lock');
                Logger::log('wpis-cron', '[CRON] ' . strtoupper($this->action) . ' - LOCK AUTO REMOVED DUE TO AN ERROR');
            }

            Logger::log('wpis-cron', '[CRON] ' . strtoupper($this->action) . '  ERROR : ' . json_encode($errstr));

            throw new \Exception($errstr);
        }
    }

    public function checkAuthorisation() {

        if( ! SettingsManager::get('enableCrons') ){
            Logger::log('wpis-cron', '[CRON] CRON ARE DISABLED');
            $this->error = true;
            return $this->response(['success' => false, 'message' => 'Cron are disabled.']);
        }

        $token = SettingsManager::get('cronKey');

        if( !isset($_GET['token']) || $_GET['token'] !== $token ){
            Logger::log('wpis-cron', '[CRON] INVALID TOKEN ' . (isset($_GET['token']) ? $_GET['token'] : ''));
            $this->error = true;
            return $this->response(['success' => false, 'message' => 'Invalid token.']);
        }

        if( isset($_GET['action']) ){
            if( !in_array($_GET['action'], $this->allowedActions) ){
                Logger::log('wpis-cron', '[CRON] INVALID ACTION ' . (isset($_GET['action']) ? $_GET['action'] : ''));
                $this->error = true;
                return $this->response(['success' => false, 'message' => 'Invalid action.']);
            }
            $this->action = $_GET['action'];
        }
        else {
            $this->action = 'auto';
        }

    }

    private function import() {

        $SyncManager = SyncManager::init();
        $result = $SyncManager->next($this->importSettings);
        $importLimit = self::IMPORT_LIMIT;

        if( $result['success'] && ! $result['isComplete'] && $this->currentImport < self::IMPORT_LIMIT ){
            Logger::log('wpis-cron', '[CRON] IMPORT - ' . "postId: {$result['lastImport']['postId']}, estateId: {$result['lastImport']['estateId']}, time: {$result['time']} ({$this->currentImport}/{$importLimit})");
            $this->currentImport++;

            $this->importList[] = [
                'postId' => $result['lastImport']['postId'],
                'estateId' => $result['lastImport']['estateId'],
                'time' => $result['time']
            ];

            $this->import();
        }
        else {
            if( !$result['success'] ){
                Logger::log('wpis-cron', '[CRON] IMPORT - ' . $result['message']);
                return [
                    'status'  => 'KO',
                    'message' => $result['message']
                ];
            }
            if( $this->action === 'import' && isset($result['lastImport'])){
                Logger::log('wpis-cron', '[CRON] IMPORT - ' . "postId: {$result['lastImport']['postId']}, estateId: {$result['lastImport']['estateId']}, time: {$result['time']} ({$this->currentImport}/{$importLimit})");
            }
            if( $result['isComplete'] ){
                $SyncManager->cache->flush();
                Logger::log('wpis-cron', '[CRON] IMPORT - Nothing to import');
            }

            Logger::log('wpis-cron', '[CRON] IMPORT - END');

        }


        return [
            'status'      => 'OK',
            'importCount' => count($this->importList),
            'importList'  => $this->importList,
        ];
    }

    private function update() {

        $SyncManager = SyncManager::init();
        $result = $SyncManager->updateStart();
        $SyncManager->cache->flush();

        if( isset($result['success']) && !$result['success']){
            Logger::log('wpis-cron', '[CRON] UPDATE - ERROR : ' . $result['message']);
            Logger::log('wpis-cron', '[CRON] UPDATE - END');
            return [
                'status' => 'KO',
                'message' => $result['message']
            ];
        }
        else {
            Logger::log('wpis-cron', '[CRON] UPDATE - estatesUpdatedCount: ' . $result['estatesUpdatedCount'] . ', time: ' . $result['time']);
            Logger::log('wpis-cron', '[CRON] UPDATE - END');
        }

        return [
            'status'      => 'OK',
            'updateCount' => $result['estatesUpdatedCount'],
            'time'        => $result['time']
        ];
    }

    private function remove() {
        $i = 1;
        $start = microtime(true);
        $SyncManager = SyncManager::init();
        $result = $SyncManager->removeInactives();

        if( isset($result['success']) && !$result['success']){
            Logger::log('wpis-cron', '[CRON] REMOVE - ERROR : ' . $result['message']);
            Logger::log('wpis-cron', '[CRON] REMOVE - END');
            return [
                'status'  => 'KO',
                'message' => $result['message']
            ];
        }

        $total = count($result['estatesRemovedList']);
        $removedList = [];
        foreach( $result['estatesRemovedList'] as $estate){
            Logger::log('wpis-cron', '[CRON] REMOVE - ' . "postId: {$estate->postId}, estateId: {$estate->estateId}  ({$i}/{$total})");
            $removedList[] = [
                'postId' =>  $estate->postId,
                'estateId' =>  $estate->estateId,
            ];
            $i++;
        }

        Logger::log('wpis-cron', '[CRON] REMOVE - END');

        $SyncManager->cache->flush();
        $elapsedTime = round(microtime(true) - $start, 3);

        return [
            'status'      => 'OK',
            'removeCount' => $i -  1,
            'removeList'  => $removedList,
            'time'        => $elapsedTime
        ];
    }

    private function response( Array $response ) {
        header('Content-Type: application/json');
        echo json_encode( $response );
    }
}

new CronManager();