<?php

namespace WPIS;

use DateTime;

use WPIS\ImmosyncClient;

require_once 'PostTypesManager.php';
require_once __DIR__ . '/../utilities/CacheManager.php';

require_once  __DIR__ . '/../../../../wp-admin/includes/image.php';

class SyncManager {

    private $postTypeProjectName;
    private $postTypeAgentName;
    private $debug;

    private $enableImportSystem = false;
    private $enableImportCustomPostType = false;
    private $enableProjectManagement = false;
    private $enableProjectManagementCustomPostType = false;
    private $enableAgentManagement = false;
    private $enableAgentManagementCustomPostType = false;

    private $enableTaxonomies = false;
    private $autoRemoveUpdatedEstates = false;
    private $expiredLockTime = 1800;
    private $cacheDirectory;
    public  $cache;

    static public function init() {

        error_reporting(E_ALL);
        ini_set('display_errors', 1);

        $class = new SyncManager();

        $class->cacheDirectory = wp_get_upload_dir()['basedir'] . '/wp-immo-sync';

        if( ! is_dir($class->cacheDirectory) ){
            mkdir($class->cacheDirectory);
        }

        $class->debug = SettingsManager::get('debug', false);

        $class->cache = new CacheManager($class->cacheDirectory . '/' . 'cache');

        $class->wp_db = $GLOBALS['wpdb'];

        $class->cacheFile = $class->cacheDirectory . '/wpis-cache.php';
        $class->lockFile = $class->cacheDirectory . '/wpis.lock';

        $class->checkSettings();

        if( $class->enableImportSystem ){
            $class->postTypeEstateName =
                $class->enableImportCustomPostType ?
                SettingsManager::get('customPostType') :
                EstatePostType::$postTypeName;
        }

        if( $class->enableProjectManagement ){
            $class->postTypeProjectName =
                $class->enableProjectManagementCustomPostType ?
                SettingsManager::get('projectCustomPostType') :
                ProjectPostType::$postTypeName;
        }

        if( $class->enableAgentManagement ){
            $class->postTypeAgentName =
                $class->enableAgentManagementCustomPostType ?
                SettingsManager::get('agentCustomPostType') :
                AgentPostType::$postTypeName;
        }

        return $class;
    }

    private function checkSettings(){
        $this->enableImportSystem                    = SettingsManager::get('enableImportSystem');
        $this->enableImportCustomPostType            = SettingsManager::get('enableImportCustomPostType');
        $this->autoRemoveUpdatedEstates              = SettingsManager::get('autoRemoveUpdatedEstates');
        $this->enableTaxonomies                      = SettingsManager::get('enableTaxonomies');
        $this->enableProjectManagement               = SettingsManager::get('enableProjectManagement');
        $this->enableProjectManagementCustomPostType = SettingsManager::get('enableProjectManagementCustomPostType');
        $this->enableAgentManagement                 = SettingsManager::get('enableAgentManagement');
        $this->enableAgentManagementCustomPostType   = SettingsManager::get('enableAgentManagementCustomPostType');
    }

    public function checking() : array {

        Logger::log('wpis-import','[IMPORT] CHECKING START');
        $limit = 1000;
        $start = microtime(true);

        $request = ImmosyncClient::init()->request('GET', 'estates', ['limit' => $limit]);

        if( isset($request->data->total) && json_decode($request->data)->total === 0 ){
            return [
                'success' => false,
                'message' =>'There is no estate to import.',
                'code' => 1000
            ];
        }

        if(  $request->code !== 200 || !json_decode($request->data)->data){
            Logger::log('wpis-import','[IMPORT] CHECKING ERROR');
            Logger::log('wpis-import','[IMPORT] CODE : ' . $request->code);
            Logger::log('wpis-import','[IMPORT] RESPONSE : ' . $request->data);
            return [
                'success' => false,
                'message' =>'An error occured while retrieving estates.',
                'code' => 1001,
                'data' => $request->data
            ];
        }

        $response = json_decode($request->data);
        $estates = $response->data;

        $query =
        'SELECT ' .
        '   id as postId, ' .
        '   meta_value as estateId, '.
        '   (SELECT meta_value FROM ' . $this->wp_db->prefix . 'postmeta WHERE ' . $this->wp_db->prefix . 'postmeta.post_id = ' . $this->wp_db->prefix . 'posts.id AND meta_key = "wpis_locked" LIMIT 1) as locked, '.
        '   (SELECT meta_value FROM ' . $this->wp_db->prefix . 'postmeta WHERE ' . $this->wp_db->prefix . 'postmeta.post_id = ' . $this->wp_db->prefix . 'posts.id AND meta_key = "wpis_undeletable" LIMIT 1) as undeletable, '.
        '   (SELECT CASE WHEN meta_value IS NULL OR meta_value = "" THEN false ELSE true END FROM ' . $this->wp_db->prefix . 'postmeta WHERE ' . $this->wp_db->prefix . 'postmeta.post_id = ' . $this->wp_db->prefix . 'posts.id AND meta_key = "wpis_signatureChanged" LIMIT 1) as signatureChanged '.
        'FROM ' . $this->wp_db->prefix . 'posts ' .
        'JOIN ' . $this->wp_db->prefix . 'postmeta ON ' . $this->wp_db->prefix . 'postmeta.post_id = ' . $this->wp_db->prefix . 'posts.id ' .
        'WHERE post_type IN ("' . $this->postTypeEstateName . '", "' . $this->postTypeProjectName . '") ' .
        'AND meta_key = "' . PostTypesManager::$postTypeMetaPrefix . '_id"';

        $estatesImported = $this->wp_db->get_results($query);
        $estatesImportedMap = [];

        // Map estate imported in associative array
        foreach($estatesImported as $estateImported){
            $estatesImportedMap[$estateImported->estateId] = $estateImported;
        }

        $totalImported = count($estatesImportedMap);
        $totalToImport = $response->total;
        $estatesMapped = [];

        // Map data from API with imported estates
        foreach( $estates as $estate ){
            $estatesMapped[] = [
                'id'             => $estate->id,
                'imported'       => isset($estatesImportedMap[$estate->id]),
                'locked'         => isset($estatesImportedMap[$estate->id]) && $estatesImportedMap[$estate->id]->locked,
                'hasBeenUpdated' => isset($estatesImportedMap[$estate->id]) && $estatesImportedMap[$estate->id]->signatureChanged,
                'data'           => $estate
            ];
        }

        // Create cache
        $cache = ['estates'  => $estatesMapped];
        $this->cache->set('data', $cache);

        // Clear estates imported of from estates from api to determine wich one should be removed
        foreach($estatesImportedMap as $estateId => $estate ){
            foreach($estatesMapped as $estateFromApi ){

                if( $estate->locked || $estate->undeletable )
                    unset($estatesImportedMap[$estateId]);

                if( $this->autoRemoveUpdatedEstates && $estateFromApi['hasBeenUpdated'] )
                    continue;

                if( (string) $estateId === (string) $estateFromApi['id'] )
                    unset($estatesImportedMap[$estateId]);

            }
        }

        $estateToRemove = $estatesImportedMap;

        // Generate inactive list
        if( !empty($estateToRemove) ){
            foreach( $estateToRemove as $key => $estate ){
                $type = get_post_meta( $estate->postId , PostTypesManager::$postTypeMetaPrefix . '_' . 'category_label' , true );
                $category = get_post_meta( $estate->postId , PostTypesManager::$postTypeMetaPrefix . '_' . 'subcategory_label' , true );
                $purpose = get_post_meta( $estate->postId, PostTypesManager::$postTypeMetaPrefix . '_' . 'purpose_label', true );
                $typeCategory = Tools::ucfirstUtf8($type) . ' / ' . Tools::ucfirstUtf8($category);
                $price = round(intval(get_post_meta( $estate->postId , PostTypesManager::$postTypeMetaPrefix . '_' . 'finance_price' , true )));
                $price = number_format_i18n($price) . ' €';
                $price = str_replace(',', ' ', $price);

                $estateToRemove[$key]->postTitle = get_post_meta( $estate->postId, PostTypesManager::$postTypeMetaPrefix . '_' . 'name', true );
                $estateToRemove[$key]->reference = get_post_meta( $estate->postId, PostTypesManager::$postTypeMetaPrefix . '_' . 'reference', true );
                $estateToRemove[$key]->price = $price;
                $estateToRemove[$key]->category = $typeCategory;
                $estateToRemove[$key]->purpose = Tools::ucfirstUtf8($purpose);
                $estateToRemove[$key]->postDateImported = get_the_time('d-m-Y', $estate->postId);
            }
        }

        Logger::log('wpis-import','[IMPORT] CHECKING RESULT : imported: ' . $totalImported . ', total: ' . $totalToImport . ', inactives: ' . count($estateToRemove) . ', time: ' . round(microtime(true) - $start, 3) );

        // Get last import date time;
        $lastUpdateAt = get_option( 'wpis_last_update', false );

        return [
            'success'         => true,
            'totalImported'   => $totalImported,
            'estateTotal'     => $totalToImport,
            'estatesInactive' => $estateToRemove,
            'lastUpdateAt'    => $lastUpdateAt
        ];
    }

    public function next( $settings ){

        if( $this->isLocked() ){
            return [
                'success' => false,
                'message' => 'An import is currently running',
            ];
        }

        $this->lock();

        if( $this->cache->get('data') === false ){
            $this->checking();
        }

        // Initialise status
        if( ! $cachedData = $this->cache->get('data') ){

            $this->unlock();
            Logger::log('wpis-import','[IMPORT][ERROR] The cache is empty.');

            return [
                'success'           => false,
                'locked'            => false,
            ];
        }

        if( ! isset($cachedData['estates']) && ! is_array($cachedData['estates']) ){

            $this->unlock();
            Logger::log('wpis-import','[IMPORT][ERROR] Estates cached are corrupted : ' . json_encode($cachedData));

            return [
                'success'           => false,
                'locked'            => false,
            ];
        }

        $estateImported = 0;
        $totalEstates = count($cachedData['estates']);
        $startTime = microtime(true);
        $isComplete = false;

        // Count how many estates are imported
        foreach( $cachedData['estates'] as $index => $estate ){
            if( $estate['imported'] ){
                $estateImported++;
            }
        }

        // Determine estate to import or update
        $targetEstate = null;
        foreach( $cachedData['estates'] as $index => $estate ){
            if( !$estate['imported'] ){
                $targetIndex = $index;
                $targetEstate = $estate;
                break;
            }
        }

        if( ! is_null($targetEstate) ){
            // Import or update targeted estate
            $forceImportImages = isset($settings->forceImportImages) ? $settings->forceImportImages : false;
            $origin = isset($settings->origin) ? $settings->origin : 'admin';

            if( ! $lastImport = $this->importEstate( $targetEstate['data'], false, $forceImportImages, $origin ) )
            {
                $this->unlock();
                return [
                    'success' => false ,
                    'message' => 'An error occured while importing estates.'
                ];
            }

            // Update cache data
            $targetEstate['imported'] = true;
            $targetEstate['locked'] = false;

            $cachedData['estates'][$targetIndex] = $targetEstate;

            $this->cache->set('data', $cachedData);
        }
        else {
            $isComplete = true;
            $lastImport = null;
        }

        $this->unlock();

        $response = [
            'success'           => true,
            'locked'            => false,
            'isComplete'        => $isComplete,
            'time'              => round(microtime(true) - $startTime, 3),
            'lastImport'        => $lastImport,
            'estateTotal'       => $totalEstates,
            'estateImported'    => $estateImported,
        ];


        return $response;
    }

    public function updateStart() : array {

        $start = microtime(true);

        if( $this->isLocked() ){
            return [
                'success' => false,
                'message' => 'An import is currently running',
            ];
        }

        $this->lock();
        $this->checking();

        $data = $this->cache->get('data');

        if( is_null( $data['estates'] ) ){
            $elapsedTime = round(microtime(true) - $start, 3);
            Logger::log('wpis-import','[IMPORT] ERROR NO ESTATES AVAILABLES');
            return [
                'success' => false,
                'message' => 'An error occured while reading cache.',
                'time' => $elapsedTime,
            ];
        }

        $progress = [
            'current' => 0,
            'total' => count($data['estates']),
            'isProcessing' => true
        ];

        foreach( $data['estates'] as $index => $estate ){
            if( $this->importEstate( $estate['data'], true ) ){
                $progress['current'] = $progress['current'] + 1;
                $this->cache->set('progress', $progress );
            }
        }

        $progress['isProcessing'] = false;

        $this->cache->set('progress', $progress );
        $this->cache->delete('data');

        $elapsedTime = round(microtime(true) - $start, 3);

        $this->unlock();
        Logger::log('wpis-import','[IMPORT] UPDATE estatesUpdatedCount: ' . $progress['current'] . ', time: ' . $elapsedTime );

        return [
            'success' => true,
            'time' => $elapsedTime,
            'estatesUpdatedCount' => $progress['current']
        ];

    }

    public function updateProgress() {

        $progress = $this->cache->get('progress');

        if( $progress ){
            if( !$progress['isProcessing'] || $progress['current'] === $progress['total'] ){
                $this->cache->delete('progress');
            }
        }

        return $progress;
    }

    public function importEstate( $estate, $updateOnly = false, $forceImportImages = false ){

        $start = microtime(true);
        $isUpdate = false;
        $postFound = $this->wp_db->get_results("SELECT post_id FROM " . $this->wp_db->prefix . "postmeta WHERE meta_key = 'wpis_id' AND meta_value = '" . $estate->id . "' LIMIT 1");

        $defaultLanguage = $estate->properties->language;
        $hasMultilanguage = $estate->properties->multilanguage;

        $estateName = $hasMultilanguage ? $estate->name->{$defaultLanguage} : $estate->name;
        $estateDesc = $hasMultilanguage ? $estate->description->base->{$defaultLanguage} : $estate->description->base;

        if( !empty( $postFound ) ){
            // UPDATE
            $isUpdate = true;
            $postId = $postFound[0]->post_id;
            $post = [
                'ID'           => $postId,
                'post_title'   => $estateName,
                'post_content' => $estateDesc,
            ];

            wp_update_post( $post );

            if( $forceImportImages ){
                $this->importPictures($estate, $postId);
            }
        }
        elseif( !$updateOnly ) {
            // Create
            $postId = wp_insert_post([
                'post_type'      => $this->postTypeEstateName,
                'post_name'      => $estate->id . '-' . $estateName,
                'post_title'     => $estateName,
                'post_content'   => $estateDesc ?? " ",
                'post_status'    => 'publish',
                'comment_status' => 'closed',
                'ping_status'    => 'closed'
            ]);

            update_post_meta( $postId, PostTypesManager::$postTypeMetaPrefix . '_locked',  0);
            update_post_meta( $postId, PostTypesManager::$postTypeMetaPrefix . '_id',  $estate->id);

            $this->importPictures($estate, $postId);
        }
        else {
            return false;
        }

        // Map multilanguage properties
        foreach( PostTypesManager::$postTypeMetas as $group){
            foreach($group['fields'] as $property){

                $propertySegments = explode('.', $property['reference']);
                $postMetaName = PostTypesManager::$postTypeMetaPrefix . '_' . str_replace('.', '_', $property['reference']);
                $translatable = $property['translatable'] ?? false;

                if( property_exists($estate, $propertySegments[0]) ){


                    if( count($propertySegments) > 1 ){
                        if( is_object($estate->{$propertySegments[0]}) && property_exists($estate->{$propertySegments[0]}, $propertySegments[1]) ){
                            $value = $estate->{$propertySegments[0]}->{$propertySegments[1]};
                        }
                    }
                    else {
                        $value = $estate->{$propertySegments[0]};
                    }

                    if( $translatable ){
                        if( $hasMultilanguage ){
                            if( isset($value->{$defaultLanguage})){
                                update_post_meta( $postId, $postMetaName,  $value->{$defaultLanguage});
                            }
                            foreach( PostTypesManager::$supportedLanguages as $language ){
                                update_post_meta( $postId, $postMetaName . '_' . $language,  $value->{$language} ?? '');
                            }
                        }
                        else {
                            foreach( PostTypesManager::$supportedLanguages as $language ){
                                delete_post_meta( $postId, $postMetaName . '_' . $language);
                            }
                            update_post_meta( $postId, $postMetaName,  $value);
                        }
                    }
                    else {
                        update_post_meta( $postId, $postMetaName,  $value);
                    }
                }
            }
        }

        // Generate signature
        $newSignature = md5(json_encode($estate->pictures));
        $oldSignature = get_post_meta($postId, PostTypesManager::$postTypeMetaPrefix . '_signature', true);

        if( get_post_meta($postId, PostTypesManager::$postTypeMetaPrefix . '_signatureChanged', true) !== '1' ){
            update_post_meta( $postId, PostTypesManager::$postTypeMetaPrefix . '_signatureChanged' , $oldSignature && $newSignature !== $oldSignature );
        }

        update_post_meta( $postId, PostTypesManager::$postTypeMetaPrefix . '_signature' ,  $newSignature );

        // Taxonomies mapping
        if( $this->enableTaxonomies ){
            // > estate_purposes
            $estatePurposeLabel = get_post_meta($postId, 'wpis_purpose_label', true);
            if( ! $term = term_exists( $estatePurposeLabel, 'estate_purposes' ) ){
                $term = wp_insert_term( $estatePurposeLabel, 'estate_purposes');
            }
            if( !is_wp_error($term) ){
                wp_set_post_terms( $postId, $term['term_id'], 'estate_purposes');
            }

            // > estate_cities
            $estateCity = get_post_meta($postId, 'wpis_address_city', true);
            if( ! $term = term_exists( $estateCity, 'estate_cities' ) ){
                $term = wp_insert_term( $estateCity, 'estate_cities');
            }
            if( !is_wp_error($term) ){
                wp_set_post_terms( $postId, $term['term_id'], 'estate_cities');
            }

            // > estate_categories
            $estateCategory = get_post_meta($postId, 'wpis_category_label', true);
            if( ! $term = term_exists( $estateCategory, 'estate_categories' ) ){
                $term = wp_insert_term( ucfirst($estateCategory), 'estate_categories');
            }
            if( !is_wp_error($term) ){
                wp_set_post_terms( $postId, $term['term_id'], 'estate_categories');
            }

            // > estate_ameneties
            $estateAmenetiesGeneric = get_post_meta($postId, 'wpis_ameneties_generic', true);
            $estateAmenetiesCustom = get_post_meta($postId, 'wpis_ameneties_custom', true);

            $estateAmenetiesGeneric = is_array($estateAmenetiesGeneric) ? $estateAmenetiesGeneric : [];
            $estateAmenetiesCustom = is_array($estateAmenetiesCustom) ? $estateAmenetiesCustom : [];

            $estateAmeneties = array_merge($estateAmenetiesGeneric, $estateAmenetiesCustom);
            $amenetiesTermIds = [];

            foreach( $estateAmeneties as $ammenety ){
                if( ! $term = term_exists( $ammenety, 'estate_ammeneties' ) ){
                    $term = wp_insert_term( $ammenety, 'estate_ammeneties' );
                }
                if( !is_wp_error($term) ){
                    $amenetiesTermIds[] = (int) $term['term_id'];
                }
            }

            wp_set_post_terms( $postId, $amenetiesTermIds, 'estate_ammeneties');
        }

        // Split estate and project if project management enabled
        if( $this->enableProjectManagement ){
            if( get_post_meta($postId, 'wpis_properties_hasChildren', true ) ) {

                wp_update_post(['ID' => $postId, 'post_type' => $this->postTypeProjectName]);

                $estateId = get_post_meta($postId, 'wpis_id', true );
                $childPosts = query_posts([
                    'post_type' => $this->postTypeEstateName,
                    'posts_per_page' => -1,
                    'meta_query' => [
                        [
                          'key'     => PostTypesManager::$postTypeMetaPrefix . '_' . 'properties_parentId',
                          'compare' => '=',
                          'value'   => $estateId,
                          'type'    => 'CHAR',
                        ]
                    ]
                ]);

                foreach( $childPosts as $childPost ){
                    wp_update_post(['ID' => $childPost->ID, 'post_parent' => $postId]);
                }

            }
        }

        // Import agent if agent management enabled
        if( $this->enableAgentManagement ){
            if( get_post_meta($postId, 'wpis_agent_firstname', true ) ) {

                // wp_update_post(['ID' => $postId, 'post_type' => $this->postTypeProjectName]);

                $agentFirstname = get_post_meta($postId, 'wpis_agent_firstname', true );
                $agentLastname  = get_post_meta($postId, 'wpis_agent_lastname', true );
                $agentPicture   = get_post_meta($postId, 'wpis_agent_picture', true );
                $agentEmail     = get_post_meta($postId, 'wpis_agent_email', true );
                $agentPhone     = get_post_meta($postId, 'wpis_agent_phone', true );

                $agentPost = get_posts([
                    'post_type'  => $this->postTypeAgentName,
                    'title' => $agentFirstname . ' ' . $agentLastname,
                ]);

                $agentPostId = isset($agentPost[0]) ? $agentPost[0]->ID : 0;

                $agentPostId = wp_insert_post([
                    'ID' => $agentPostId,
                    'post_title' => $agentFirstname . ' ' . $agentLastname,
                    'post_status' => 'publish',
                    'post_type' => $this->postTypeAgentName
                ]);

                update_post_meta($agentPostId, 'wpis_agent_picture', $agentPicture, true );
                update_post_meta($agentPostId, 'wpis_agent_firstname', $agentFirstname, true );
                update_post_meta($agentPostId, 'wpis_agent_lastname', $agentLastname, true );
                update_post_meta($agentPostId, 'wpis_agent_email', $agentEmail, true );
                update_post_meta($agentPostId, 'wpis_agent_phone', $agentPhone, true );

            }
        }

        // Post Update
        if( file_exists(get_stylesheet_directory() . '/wpis-post-update.php' ) ){
            require_once get_stylesheet_directory() . '/wpis-post-update.php';
            if( function_exists('wpis_post_update') ) {
                wpis_post_update($postId);
            }
        }

        // Logging
        if( $postId ){
            Logger::log('wpis-import','[IMPORT] ' . ($isUpdate ? 'UPDATE' : 'CREATE' ) . ' postId: ' . $postId . ', estateId: ' . $estate->id . ', imageCount: ' . count($estate->pictures) . ', time: ' . round((microtime(true) - $start), 3 ));
        }

        $now = new DateTime('now');
        $dateTime = [
            'date' => $now->format('d/m/Y'),
            'time' => $now->format('H:i:s')
        ];

        update_option( 'wpis_last_update', $dateTime, false );
        update_post_meta( $postId, PostTypesManager::$postTypeMetaPrefix . '_completed',  1);

        return [
            'postId' => $postId,
            'estateId' => $estate->id,
            'estateName' => $estateName
        ];
    }

    public function importPictures($estate, $postId){

        $wpUploadDir = wp_upload_dir();
        $hasPostThumbnail = false;

        if( $wpUploadDir['error'] ){
            return $wpUploadDir['error'];
        }

        foreach($estate->pictures as $index => $picture ){

            $filename   = $estate->id . '_' . $picture->id . '.' . $picture->format;
            $realpath   = $wpUploadDir['path'] . '/' . $filename;
            $url        = $wpUploadDir['url'] . '/' . $filename;

            $url = $picture->urls->large;

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_HEADER, false);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);

            $data = curl_exec($ch);
            $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch) ;

            if( $code === 200 ){
                $result = file_put_contents($realpath, $data);

                $uploadId = wp_insert_attachment([
                    'guid'           => $realpath,
                    'post_mime_type' => mime_content_type( $realpath ),
                    'post_title'     => $filename,
                    'post_content'   => '',
                    'post_status'    => 'inherit',
                    'menu_order'     => $index,
                ], $realpath,  $postId, true);

                if( is_wp_error($uploadId) ){
                    Logger::log('wpis-import','[IMPORT] IMAGE INSERT ATACHMENT ERROR :');
                    Logger::log('wpis-import','[IMPORT] URL : ' . $url);
                    Logger::log('wpis-import','[IMPORT] REALPATH : ' . $realpath);
                    Logger::log('wpis-import','[IMPORT] POST_ID : ' . $postId);
                    Logger::log('wpis-import','[IMPORT] WP_ERROR : ' . json_encode($uploadId));
                }

                if( $this->debug ){
                    wp_update_attachment_metadata( $uploadId, wp_generate_attachment_metadata( $uploadId, $realpath ) );
                }
                else {
                    // phpcs:ignore WordPress.PHP.NoSilencedErrors -- Silencing notice and warning is intentional. See https://core.trac.wordpress.org/ticket/42480
                    @wp_update_attachment_metadata( $uploadId, wp_generate_attachment_metadata( $uploadId, $realpath ) );
                }

                if( !$hasPostThumbnail ){
                    set_post_thumbnail($postId, $uploadId);
                    $hasPostThumbnail = true;
                }
            }
            else {
                Logger::log('wpis-import','[IMPORT] IMAGE IMPORT ERROR : ' . $code . ' ' . $data);
                Logger::log('wpis-import','[IMPORT] URL : ' . $url);
                Logger::log('wpis-import','[IMPORT] CODE : ' . $code);
                Logger::log('wpis-import','[IMPORT] RESPONSE : ' . $data);
            }

        }

        return 'success';
    }

    public function removeInactives() : array {
        $checking = $this->checking();

        if( isset($checking['success']) && !$checking['success']){
            return $checking;
        }

        $i = 0;

        foreach($checking['estatesInactive'] as $estate ){
            $attachments = get_attached_media( '', $estate->postId );
            foreach ($attachments as $attachment) {
                wp_delete_attachment($attachment->ID, true);
            }
            wp_delete_post($estate->postId, true);
            $i++;
        }

        return [
            'success' => true,
            'estatesRemoved' => $i,
            'estatesRemovedList' => $checking['estatesInactive']
        ];
    }

    public function removeAll() : array {

        $query = new \WP_Query( [
            'post_type' => [$this->postTypeEstateName, $this->postTypeProjectName],
            'meta_key' => 'wpis_id',
            'posts_per_page' => 1000
        ]);

        $i = 0;

        if( !$query->have_posts() ){
            return [
                'success' => false,
                'message' => 'There is no estate to remove.'
            ];
        }

        if( $this->enableTaxonomies ){
            foreach( ['estate_cities', 'estate_categories', 'estate_purposes', 'estate_ammeneties'] as $taxonomy ){
                $terms = get_terms( $taxonomy, ['fields' => 'ids', 'hide_empty' => false] );
                foreach ( $terms as $value ) {
                    wp_delete_term( $value, $taxonomy );
                }
            }
        }

        foreach( $query->posts as $post ){
            $attachments = get_attached_media( '', $post->ID );
            foreach ($attachments as $attachment) {
                wp_delete_attachment($attachment->ID, true);
            }
            wp_delete_post($post->ID, true);
            $i++;
        }

        return [
            'success' => true,
            'estatesRemoved' => $i,
        ];
    }

    private function isLocked() : bool {

        if( $this->debug ) {
            return false;
        }

        if( file_exists($this->lockFile) ){

            if( (new \DateTime())->getTimestamp() - filemtime($this->lockFile) >= $this->expiredLockTime ){
                Logger::log('wpis-import','[IMPORT] LOCK FILE HAS BEEN REMOVED DUE TO IT LIFE TIME.');
                $this->unlock();
                return $this->isLocked();
            }

            return true;
        }

        return false;
    }

    private function lock() : bool {

        if( $this->debug ) {
            return false;
        }

        touch($this->lockFile);

        return true;
    }

    private function unlock() : bool {

        if( $this->debug ) {
            return true;
        }

        if( file_exists($this->lockFile) ){
            unlink($this->lockFile);
        }

        return true;
    }

}